% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Ph.D. Course "Intelligence for Embedded Systems"
% Prof. Manuel Roveri
% Politecnico di Milano, Italy
% Jan, 2015
% 
% ----- "Uncertainty, Information and Learning Mechanisms - Part 1"
% -> Example 3_1_B) Central Limit Theorem (CLT) under the Lyapunov assumption 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear
clc
close all


%% PARAMETERS %%

% Interval extremes for the perturbation U(a,b)
a = -1;
b = 1;

% Number of samples
n_sample = 10000;

% Number of dimensions
d = 25;

%% MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Random generation of the parameters of the linear function
theta = randn(d,1);

% Random generation of the perturbations U(a,b)
delta_x = (rand(n_sample,d) - 0.5) * (b-a) + (b+a)/2;

% Computation of each component of the error
Y_i = repmat(theta',n_sample,1) .* delta_x;

% Computation of the error
delta_y = sum(Y_i,2);

% Computation of true mean and variance of the error
mean_delta_y = theta' * repmat((b+a) / 2,d,1);
var_delta_y = (b-a)^2 / 12 * (theta') * theta;

%Plot real vs expected
[f xx] = hist(delta_y,40);
dx = diff(xx(1:2));
bar(xx,f/sum(f*dx), 'BarWidth', 1);
hold on;
x = min(delta_y):0.1:max(delta_y);
y = normpdf(x,mean_delta_y,sqrt(var_delta_y));
plot(x,y,'r');
